@@ -4,11 +4,11 @@ import math
 import numpy as np
 import onnxruntime as ort
 
-from PIL import Image
-
-from .utils import download_file, load_json, load_image, get_color
+from .ops import py_cpu_nms, normalize, mask_to_polygon
+from .utils import download_file, load_json, get_color
+from .utils import load_image, show_image, save_image
+from .draw import render_results
 from .video import Video
-from . import draw
 
 
 def resize(img, size):
@@ -24,11 +24,6 @@ def crop(img, size):
     return img[top:bottom, left:right]
 
 
-def normalize(img, mean, std):
-    img = (img / 255 - np.array(mean)) / np.array(std)
-    return img.astype(np.float32)
-
-
 def preprocess(img):
     img = resize(img, 256)
     img = crop(img, 224)
@@ -60,7 +55,7 @@ def prepare_input(img, shape):
     # left, top, right, bottom = round(dw - 0.1), round(dh - 0.1), round(dw + 0.1), round(dh + 0.1)
     left, top, right, bottom = 0, 0, shape[3] - size[0], shape[2] - size[1]
     img = cv2.copyMakeBorder(img, top, bottom, left, right, cv2.BORDER_CONSTANT, value=(114, 114, 114))
-    img = img.transpose(2, 0, 1).reshape(shape).astype(np.float32)
+    img = img.transpose((2, 0, 1)).reshape(shape).astype(np.float32)
     return img / 255
 
 
@@ -73,13 +68,25 @@ def iou(box1, box2):
     return intersection_area / union_area
 
 
+# def non_maximum_suppression(objects, iou_threshold):
+#     results = []
+#     objects.sort(key=lambda obj: obj['confidence'], reverse=True)
+#     while len(objects) > 0:
+#         results.append(objects[0])
+#         objects = [obj for obj in objects if iou(obj['box'], objects[0]['box']) < iou_threshold]
+    # return results
+
+
 def non_maximum_suppression(objects, iou_threshold):
-    results = []
-    objects.sort(key=lambda obj: obj['confidence'], reverse=True)
-    while len(objects) > 0:
-        results.append(objects[0])
-        objects = [obj for obj in objects if iou(obj['box'], objects[0]['box']) < iou_threshold]
-    return results
+    dets = []
+    for obj in objects:
+        s = obj['confidence']
+        x, y, w, h = obj['box']
+        dets.append([x, y, x + w, y + h, s])
+    if dets:
+        idxs = py_cpu_nms(np.array(dets), iou_threshold)
+        return [objects[idx] for idx in idxs]
+    return []
 
 
 def sigmoid_mask(z):
@@ -100,23 +107,7 @@ def get_mask(row, box, size):
     return mask
 
 
-def mask_to_polygon(mask, origin):
-    """Returns the largest bounding polygon based on the segmentation mask."""
-    contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)[0]
-    lengths = [len(contour) for contour in contours]
-    polygon = contours[np.argmax(lengths)].reshape(-1, 2)
-    polygon = polygon + np.array(origin)
-    return polygon.tolist()
-
-
-def approximate_polygon(polygon, approx=0.02):
-    contour = np.array([polygon]).astype(np.int32)
-    epsilon = approx * cv2.arcLength(contour, True)
-    contour = cv2.approxPolyDP(contour, epsilon, True)
-    return contour.reshape(-1, 2).tolist()
-
-
-def process_output(outputs, size, shape, classes, confidence, iou_threshold):
+def process_output(outputs, size, shape, classes, conf_threshold=0.25, iou_threshold=0.7, approx=0.001):
     """Converts the RAW model output from YOLOv8 to an array of detected
     objects, containing the bounding box, label and the probability.
     """
@@ -125,7 +116,6 @@ def process_output(outputs, size, shape, classes, confidence, iou_threshold):
     if len(outputs) == 2:
         output1 = outputs[1][0].astype("float")
         output1 = output1.reshape(output1.shape[0], output1.shape[1] * output1.shape[2])
-    
     img_width, img_height = size
     model_width, model_height = shape[3], shape[2]
     scale = 1 / min(model_width / img_width, model_height / img_height)
@@ -134,7 +124,7 @@ def process_output(outputs, size, shape, classes, confidence, iou_threshold):
         xc, yc, w, h = row[:4]
         probs = row[4:4+len(classes)]
         idx = probs.argmax()
-        if probs[idx] < confidence:
+        if probs[idx] < conf_threshold:
             continue
         x1, y1 = round((xc - w/2) * scale), round((yc - h/2) * scale)
         x2, y2 = round((xc + w/2) * scale), round((yc + h/2) * scale)
@@ -143,7 +133,6 @@ def process_output(outputs, size, shape, classes, confidence, iou_threshold):
             obj['mask'] = row[4+len(classes):]
         objects.append(obj)
     results = non_maximum_suppression(objects, iou_threshold)
-
     for result in results:
         if len(outputs) == 2:
             x, y, w, h = result['box']
@@ -151,7 +140,7 @@ def process_output(outputs, size, shape, classes, confidence, iou_threshold):
             size = (round(model_width * scale), round(model_height * scale))
             mask = get_mask(mask, result['box'], size)
             mask = cv2.threshold(mask, 127, 255, cv2.THRESH_BINARY)[1]
-            result['polygon'] = mask_to_polygon(mask, (x, y))
+            result['polygon'] = mask_to_polygon(mask, (x, y), approx)
             # result.pop('mask', None)
             result['mask'] = mask
     return results
@@ -171,33 +160,24 @@ def count_objects(results):
 class Model:
     def load(self, model_uri):
         config_uri = f"{os.path.splitext(model_uri)[0]}.json"
-        config_src = download_file(config_uri)
-        self.config = load_json(config_src)
-        model_src = download_file(model_uri)
-        self.session = ort.InferenceSession(model_src, providers=['CPUExecutionProvider'])
+        self.config = load_json(download_file(config_uri))
+        providers = ["CUDAExecutionProvider", "CoreMLExecutionProvider", "CPUExecutionProvider"]
+        providers = [provider for provider in ort.get_available_providers() if provider in providers] 
+        self.session = ort.InferenceSession(download_file(model_uri), providers=providers)
         self.input_name = self.session.get_inputs()[0].name
         self.input_shape = self.config['inputShape']
 
-    def run(self, img, confidence=0.25, iou_threshold=0.7):
-        img = img if isinstance(img, np.ndarray) else np.array(img.convert("RGB"))
+    def run(self, img, conf_threshold=0.25, iou_threshold=0.7, approx=0.001):
         if self.config.get('task'):
             img_size = img.shape[1], img.shape[0]
-            outputs = self.session.run(None, {self.input_name: prepare_input(img, self.input_shape)})
-            return process_output(outputs, img_size, self.input_shape, self.config['classes'], confidence, iou_threshold)
+            inputs = {self.input_name: prepare_input(img, self.input_shape)}
+            outputs = self.session.run(None, inputs)
+            return process_output(outputs, img_size, self.input_shape, self.config['classes'], conf_threshold, iou_threshold, approx)
         outputs = self.session.run(None, {self.input_name: preprocess(img)})
-        results = postprocess(outputs, self.config['classes'])
-        return results
+        return postprocess(outputs, self.config['classes'])
     
 
 def load_model(model_uri):
     model = Model()
     model.load(model_uri)
     return model
-    
-
-def render_results(img, results):
-    if isinstance(img, np.ndarray):
-        return draw.render_results(img, results)
-    img = np.array(img)
-    img = draw.render_results(img, results)
-    return Image.fromarray(img)
